/* *****************************************************************************
 *
 *  Description:  Two methods, each of which fill an array with n values.
 *
 *                > java-introcs ArrayFill 25000
 *
 *                To turn off Java optimizations, use the -Xint flag:
 *
 *                > java-introcs -Xint ArrayFill 25000
 *
 *  The -Xint option:
 *    "Runs the application in interpreted-only mode. Compilation to
 *    native code is disabled, and all bytecode is executed by the
 *    interpreter. The performance benefits offered by the
 *    just-in-time (JIT) compiler aren't present in this mode." 
 *  - https://docs.oracle.com/en/java/javase/17/docs/specs/man/java.html
 *
 **************************************************************************** */


public class ArrayFill {

    // Experiment 1: return an array of n ints
    public static int[] fill1(int n) {
        // allocate an array of size 1
        int[] filled = { 2024 };

        for (int i = 1; i < n; i++) {
            // copy each element from filled array to copy array
            int[] copy = new int[i + 1];           // allocate array of size i+1
            for (int j = 0; j < filled.length; j++)  // copy i elements from filled
                copy[j] = filled[j];

            copy[i] = 2024;                        // set last element of copy
            filled = copy;
        }
        return filled;

    }

    // Experiment 2: return an array of n ints
    public static int[] fill2(int n) {
        int[] filled = new int[n];
        for (int i = 0; i < n; i++) {
            filled[i] = 2024;
        }
        return filled;

    }

    public static void main(String[] args) {

        int n = Integer.parseInt(args[0]);
        double startTime, endTime;
        int[] fill1, fill2;

        // timing fill1()
        startTime = System.currentTimeMillis();
        fill1 = fill1(n);
        endTime = System.currentTimeMillis();
        StdOut.println("Elapsed time for fill1() = " +
                               (endTime - startTime) / 1000 + "s");

        // timing fill2()
        startTime = System.currentTimeMillis();
        fill2 = fill2(n);
        endTime = System.currentTimeMillis();
        StdOut.println("Elapsed time for fill2() = " +
                               (endTime - startTime) / 1000 + "s");

    }
}

