/* **********************************************************************
 * 
 * Two functions that reverse an array.
 *
 * The main program takes an integer command-line argument - which 
 * is the length of the array to reverse.
 *
 * Uses System.currentTimeMillis(), which returns the difference, measured
 * in milliseconds, between the current time and midnight, January 1, 1970
 * UTC(coordinated universal time).
 *
 * Also will count off seconds (up to 60 seconds) for long running
 * experiments.
 *
 * Examples:
 *    java-introcs Reverse 100000
 *    java-introcs Reverse 200000
 *    java-introcs Reverse 400000
 *    java-introcs Reverse 800000
 *
 * Use -Xint to turn off the Java optimizer, for example:
 *    java-introcs -Xint Reverse 100000
 *    java-introcs -Xint Reverse 200000
 *    java-introcs -Xint Reverse 400000
 *    java-introcs -Xint Reverse 800000
 * 
 *  The -Xint option:
 *    "Runs the application in interpreted-only mode. Compilation to
 *    native code is disabled, and all bytecode is executed by the
 *    interpreter. The performance benefits offered by the
 *    just-in-time (JIT) compiler aren't present in this mode." 
 *  - https://docs.oracle.com/en/java/javase/17/docs/specs/man/java.html
 *
 * **********************************************************************/

public class Reverse {
    // reverse1 function
    public static void reverse1 (int[] array) {
        int n = array.length;
        for (int i = 0; i < n/2; i++) {
            int temp = array[i];
            array[i] = array[n - 1 - i];
            array[n - 1 - i] = temp;
        }
    }

    // reverse2 function
    public static void reverse2(int[] array) {
        int n = array.length;
        for (int i = 0; i < n; i++) {
            int value = array[0];
            for (int j = 0; j < n-i-1; j++)
                array[j] = array[j+1];
            array[n-i-1] = value;
    }
    }

    
    // main method - takes one command-line argument, the length of the array
    public static void main(String[] args) {
        // create an array of n int's
        int n = Integer.parseInt(args[0]);
        int[] array = new int[n];
        for (int i = 0; i < n; i++) {
            array[i] = i;
        }

        // used for estimating run-time 
        double startTime, endTime;

        // time reverse1
        StdAudio.playInBackground("CountTo60.wav");
        startTime = System.currentTimeMillis();
        reverse1(array);
        endTime = System.currentTimeMillis();
        StdAudio.stopInBackground();
        StdOut.println("Elapsed time for reverse1() = " +
                               (endTime - startTime) / 1000 + "s");
        // time reverse2
                StdAudio.playInBackground("CountTo60.wav");        
        startTime = System.currentTimeMillis();
        reverse2(array);
        endTime = System.currentTimeMillis();
        StdOut.println("Elapsed time for reverse2() = " +
                               (endTime - startTime) / 1000 + "s");
        StdAudio.stopInBackground();

    }

}
