/* **********************************************************************
 * 
 * Two functions that shuffle an array.
 *
 * The main program takes an integer command-line argument - which 
 * is the length of the array to shuffle.
 *
 * Uses System.currentTimeMillis(), which returns the difference, measured
 * in milliseconds, between the current time and midnight, January 1, 1970
 * UTC(coordinated universal time).
 * 
 *    java-introcs Shuffle 4000
 *    java-introcs Shuffle 8000
 *    java-introcs Shuffle 16000
 *    java-introcs Shuffle 32000
 *    java-introcs Shuffle 64000
 *    java-introcs Shuffle 128000
 *
 * Use -Xint to turn off the Java optimizer, for example
 *    java-introcs -Xint Shuffle 4000
 *    java-introcs -Xint Shuffle 8000
 *    java-introcs -Xint Shuffle 16000
 *    java-introcs -Xint Shuffle 32000
 *    java-introcs -Xint Shuffle 64000
 *    java-introcs -Xint Shuffle 128000
 *
 *  The -Xint option:
 *    "Runs the application in interpreted-only mode. Compilation to
 *    native code is disabled, and all bytecode is executed by the
 *    interpreter. The performance benefits offered by the
 *    just-in-time (JIT) compiler aren't present in this mode." 
 *  - https://docs.oracle.com/en/java/javase/17/docs/specs/man/java.html
 *
 * **********************************************************************/

public class Shuffle {
    // shuffle1 function
    public static int[] shuffle1 (int[] original) {
        int n = original.length;
        int[] shuffled = new int[n];
        for (int i = 0; i < n; i++)
            shuffled[i] = original[i];
        
        for (int i = 0; i < n; i++) {
            int r = StdRandom.uniformInt(i + 1);
            int temp = shuffled[r];
            shuffled[r] = shuffled[i];
            shuffled[i] = temp;
        }
        return shuffled;
    }

    // shuffle2 function
    public static int[] shuffle2 (int[] original) {
        int n = original.length;
        int[] shuffled = new int[n];
        for (int i = 0; i < n; i++) {
            int r = StdRandom.uniformInt(n - i);
            int temp = original[r];
            shuffled[i] = temp;
            for (int j = r; j < n - 1; j++)
                original[j] = original[j + 1];
            
        }
        return shuffled;
    }
    
    // main method - takes one command-line argument, the length of the array
    public static void main(String[] args) {
        // create an array of n int's
        int n = Integer.parseInt(args[0]);
        int[] array = new int[n];
        for (int i = 0; i < n; i++) {
            array[i] = StdRandom.uniformInt(100);
        }

        // used for estimating run-time 
        double startTime, endTime;

        // time shuffle1
        startTime = System.currentTimeMillis();
        shuffle1(array);
        endTime = System.currentTimeMillis();
        StdOut.println("Elapsed time for shuffle1() = " +
                               (endTime - startTime) / 1000 + "s");
        // time shuffle2
        startTime = System.currentTimeMillis();
        shuffle2(array);
        endTime = System.currentTimeMillis();
        StdOut.println("Elapsed time for shuffle2() = " +
                               (endTime - startTime) / 1000 + "s");
        

    }

}
